/*
STATA program for the g-estimation of accelerated failure-time models for control of the HWSE (gest) by Jonathan Chevrier, PhD 
is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License. This license allows you to use, 
share and modify the program for noncommercial purposes only as long as it is redistributed under the same license and attribution 
to this work is made according to the license rules. We also request that you cite the paper from which this work was derived: 

Chevrier J, Picciotto S, Eisen EA. A comparison of standard methods with g-estimation of accelerated failure-time models to address
the healthy worker survivor effect: Application in a cohort of autoworkers exposed to metalworking fluids. Epidemiology, 23(2): 212-219.

Downloading, using, sharing and/or modifying this program constitutes acceptance of these terms. Code is provided "as is" without warranty.
User must review the code and assumes all responsibility and liability.

(C) 2011 by Jonathan Chevrier, PhD

*/



capture program drop gest
program gest, rclass
version 11.2


syntax varlist(min=2 max=2 numeric), COVars(varlist numeric) Range(numlist min=2 max=2) COUNTexp(numlist min=1 max=1) /// 
ACTive(varname numeric) id(varname numeric) Follow(varname numeric) Time(varname numeric) BOOTstraps(integer) ci(name) /// 
[cens(varname numeric) CENSPred(varlist numeric) stat(name) SAVing(string asis)]

display as text "Analysis Progress:"
display " "
display as text "Computing G Estimand"
display " "

*************************************
*** SET LOCAL MACROS AND TEMPVARS ***
*************************************
capture { 
*REMOVE STAR TO SUPPRESS OUTPUT
*** TOKENIZE
tokenize `varlist'
local outcome `1'
local exposure `2'

local start: word 1 of `range'
local stop: word 2 of `range'


*** SET TEMPVARS
tempvar incl temp_obs temp_incl tte MS_follow minyr maxyr dies temp_adcens adcens maxtime
tempvar tempid idclust inv_wt_loss expect t_what wt_loss countfact inter_countfact delta upsi u

*** SET TEMPFILE
tempfile temp_t0


*** COUNT PARTICIPANTS
sort `id' `follow'
count if `id'~=`id'[_n-1]
local obs = r(N)


by `id', sort: egen `MS_follow' = max(`follow')
egen `maxtime' = max(`time')
by `id', sort: egen `minyr' = min(`time')
by `id', sort: egen `maxyr' = max(`time')
by `id', sort: gen `tte' = (`maxtime'-`minyr')


if "`cens'" ~= "" {
	gen `incl' = `cens'
	recode `incl' 0=1 1=0
	by `id', sort: egen `temp_incl' = min(`incl')
	}
	else if "`cens'" == "" {
		gen `temp_incl' = 1
		}

*** IDENTIFIES ADMINISTRATIVELY CENSORED PARTICIPANTS
gen `temp_adcens' = 0

if "`cens'" ~="" {
	replace `temp_adcens' = 1 if `cens'==0 & `outcome'==0
	}
	else if "`cens'" == "" {
		replace `temp_adcens' = 1 if `outcome'==0
		}
by `id', sort: egen `adcens' = min(`temp_adcens')


***************************************
*** VERIFY VARIABLES AND DATA TYPES ***
***************************************

capture assert "`ci'"=="normal" | "`ci'"=="percentile" | "`ci'"=="bc" | "`ci'"=="bca" | "`ci'"=="all"
	if _rc~=0 {
	display as error "ci must either be: 'normal', 'percentile', 'bc', 'bca' or 'all'. See 'help estat bootstrap' for more details."
	exit 198
	}
	

capture assert `countexp' > 0
	if _rc~=0 {
	display as error "countexp must be larger than 0"
	exit 198
	}

foreach i in active id follow {
	capture assert  ``i'' ~= .
	if _rc~=0 {
		display as error "`i' cannot contain missing values"
		exit 198
		}
	}

foreach i in outcome exposure active {
	capture assert ``i'' == 0 | ``i'' == 1 | ``i'' ~= .
	if _rc~=0 {
		display as error "`i' must be coded as 0 or 1 and cannot contain missing values"
		exit 198
		}
	}

if "`cens'" ~= "" {
	capture assert `cens' == 0 | `cens' == 1 | `cens' ~= .
	if _rc~=0 {
		display as error "cens must be coded as 0 or 1 and cannot contain missing values"
		exit 198
		}
	}

	
if "`stat'" ~= "" {
	capture assert "`stat'"=="psi" | "`stat'"=="sr" | "`stat'"=="hr" | "`stat'"=="all"
	if _rc~=0 {
		display as error "stat must either be: 'psi', 'sr', 'hr' or 'all'"
		exit 198
		}
	}

capture assert `start' <= `stop'
if _rc~=0 {
	display as error "Range must be indicated as range(lowerbound upperbound)"
	exit 198
	}



***********************************************
*** DIAGNOSTIC TOOLS FOR SKIPPED BOOTSTRAPS ***
***********************************************
local n_skipped = 0
matrix input b_skipped = (0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)	

	
*****************
*** RUN LOOPS ***
*****************

local boot = `bootstraps'+1
set more off

forvalues i = 1/`boot' {
	preserve
	set seed `i'
	gen `idclust' = `id'

	if `i' == 2 {
		nois _dots 0, title(Bootstrapping) reps(`bootstraps')
		}
		
	if `i' > 1 {
		bsample, cluster(`id') idcluster(`tempid')
		replace `idclust' = `tempid'
	}

	capture drop `inv_wt_loss'
	capture drop `expect'
	capture drop `t_what'
	

	if "`cens'" ~= "" {
		xi: logistic `incl' `exposure' `censpred' if `active'==1
		capture drop `wt_loss'
		predict `wt_loss'
		sort `idclust' `follow'
		capture drop `inv_wt_loss'
		gsort +`idclust' -`follow'
		by `idclust': gen `inv_wt_loss' = exp(sum(log(`wt_loss')))
		replace `inv_wt_loss' = 1/`inv_wt_loss'
		}

	if "`cens'" == "" {
		gen `inv_wt_loss' = 1
		}

		xi: logistic `exposure' `covars' `tte' if `active'==1 & `temp_incl'==1, vce(cluster `id')
		predict `expect'
	
	*** GENERATE SCALARS FOR WEIGHTS DESCRIPTIVE DATA
	if `i' == 1 {
		sum `inv_wt_loss'
		return scalar m_wts = r(mean)
		return scalar sd_wts = r(sd)
		return scalar min_wts = r(min)
		return scalar max_wts = r(max)
		}
	
	local min = `start'
	local max = `stop'
	local first = 0
	local oops = 0
	local upsi_l = 0
	local upsi_m = 0
	local upsi_h = 0
	local num = 0


		local mid = `min' + ((`max'-`min')/2)
		foreach psi in `min' `mid' `max' {
		capture drop `countfact'
		capture drop `inter_countfact'
		capture drop `delta'
		capture drop `upsi'
		capture drop `u'
		local num = `num'+1

		gen `inter_countfact' = exp(`exposure'*`psi')
		by `idclust', sort: egen `countfact' = sum(`inter_countfact')

			if `psi' >=0 {
				gen `delta' = 1 if `countfact' < ceil(`tte')
				replace `delta' = 0 if (`countfact' >= ceil(`tte') | `adcens'==1) & `tte'~=.
				}
			else if `psi' <0 {
				gen `delta' = 1 if `countfact'*exp(-1*`psi') < ceil(`tte')
				replace `delta' = 0 if (`countfact'*exp(-1*`psi') >= ceil(`tte') | `adcens'==1) & `tte'~=. 
				}
				
		*xi: logistic `exposure' `delta' `covars' `tte' if `active'==1 & `temp_incl'==1, vce(cluster `id')
				
		gen `u' = `inv_wt_loss'*`delta'*(`exposure' - `expect') if `active'==1 & `temp_incl'==1
		egen `upsi' = total(`u')
		sum `u'
		display "Min = "`min' " Mid = " `mid' " Max = " `max'
		display "Current psi = " `psi'
		display "Num = " `num'
		
			if `num' == 1 {
				local upsi_l = r(mean)
				}
			else if `num' == 2 {
				local upsi_m = r(mean)
				}
			else if `num' == 3 {
				local upsi_h = r(mean)
				
				if (`upsi_m' < 0 & `upsi_l' > 0) | (`upsi_m' > 0 & `upsi_l' < 0) {
					local max = `mid'
					}
				else if (`upsi_m' > 0 & `upsi_h' < 0) | (`upsi_m' < 0 & `upsi_h' > 0) {
					local min = `mid'
					}
				}
			}

	if ((`upsi_h' < 0 & `upsi_l' < 0) | (`upsi_h' > 0 & `upsi_l' > 0)) {
		display "Bounds for psi too narrow"
		local n_skipped = `n_skipped'+1
		matrix b_skipped[1, `n_skipped'] = `i'
		restore
		continue
		}

	if `upsi_h' == 0 | `upsi_l' ==0 | `upsi_m' ==0 {
		display "All deltas = 0, Adjust bounds"
		restore
		continue, break
		}
		
	while ((`max' - `min') > 0.0001) {
		local mid = `min' + ((`max'-`min')/2)
		local psi = `min' + ((`max'-`min')/2)
		
		capture drop `countfact'
		capture drop `inter_countfact'
		capture drop `delta'
		capture drop `upsi'
		capture drop `u'
		local num = `num'+1

		gen `inter_countfact' = exp(`exposure'*`psi')
		by `idclust', sort: egen `countfact' = sum(`inter_countfact')
		
			if `psi' >=0 {
				gen `delta' = 1 if `countfact' < ceil(`tte')
				replace `delta' = 0 if (`countfact' >= ceil(`tte') | `adcens'==1) & `tte'~=.
				}
			else if `psi' <0 {
				gen `delta' = 1 if `countfact'*exp(-1*`psi'*`tte') < ceil(`tte')
				replace `delta' = 0 if (`countfact'*exp(-1*`psi'*`tte') >= ceil(`tte') | `adcens'==1) & `tte'~=. 
				}
				
		*xi: logistic `exposure' `delta' `covars' `tte' if `active'==1 & `temp_incl'==1, vce(cluster `id')
		
		gen `u' = `inv_wt_loss'*`delta'*(`exposure' - `expect') if `active'==1 & `temp_incl'==1
		egen `upsi' = total(`u')
		sum `u'
		display "Min = "`min' " Mid = " `mid' " Max = " `max'
		display "Current psi = " `psi'
		display "Num = " `num'	
		
		local upsi_m = r(mean)
		
		if (`upsi_m' < 0 & `upsi_l' > 0) | (`upsi_m' > 0 & `upsi_l' < 0) {
			local max = `mid'
			}
		else if (`upsi_m' > 0 & `upsi_h' < 0) | (`upsi_m' < 0 & `upsi_h' > 0) {
			local min = `mid'
			}
	}
	
sum `u'
scalar psi_`i' = `mid'
scalar upsi_`i' = r(mean)


_pctile `countfact' [pw=`inv_wt_loss'] if `follow'==`MS_follow' & `delta'==1, p(50)
scalar med_t0_`i' = r(r1)
scalar med_ta_`i' = med_t0_`i' + `countexp' - `countexp'*exp(psi_`i')
scalar sr_add_`i' = med_ta_`i' / med_t0_`i'


	keep if `follow'==`MS_follow'
	gen `t_what' = 0
	save `temp_t0', replace
	replace `countfact' = `countfact' + `countexp' - `countexp'*exp(psi_`i')
	replace `id' = `id' + 1000000
	replace `idclust' = `idclust' + 1000000
	replace `t_what' = 1
	append using `temp_t0'
	count
	by `t_what', sort: sum `countfact'

stset `countfact' if `follow'==`MS_follow' & `temp_incl'==1, failure(`outcome')
stcox `t_what' if `follow'==`MS_follow' & `temp_incl'==1
scalar hr_add_`i' = exp(_b[`t_what'])


if `i' ==1 {
	scalar psi1 = psi_`i'
	scalar sr_add1 = sr_add_`i'
	scalar hr_add1 = hr_add_`i'
	}

display "Ending Psi = " psi_`i'
display "SR = " sr_add_`i'
display "HR = " hr_add_`i'
display "Bootstrap No. " `i'-1

if `i' > 1 {
	nois _dots `i' 0
	}

restore
}

return scalar n_skip = `n_skipped'
matrix b_skip = b_skipped
} 
*REMOVE STAR TO SUPRESS OUTPUT

if `boot' == 0 | `boot' == 1 {
	display "Psi = " psi_1
	display "SR = " sr_add_1
	display "HR = " hr_add_1
	}

if `boot' > 1 {
	preserve
	clear
	set more off
	quietly set obs `boot'
	quietly gen psi = .
	quietly gen hr = .
	quietly gen sr = .

	forvalues i = 2/`boot' {
		local c = `i'-1
		capture replace psi in `c' = psi_`i'
		capture replace hr in `c' = hr_add_`i'
		capture replace sr in `c' = sr_add_`i'
		}
	quietly drop in `boot'

	
	if "`stat'" == "psi" | "`stat'" == "all" | "`stat'" == ""{
		matrix obs = psi1
		quietly bstat psi, stat(obs) n(`obs')
		estat bootstrap, `ci'
		}

	if "`stat'" == "sr" | "`stat'" == "all" | "`stat'" == ""{
		matrix obs = sr_add1
		quietly bstat sr, stat(obs) n(`obs')
		estat bootstrap, `ci'
		}
	
	if "`stat'" == "hr" | "`stat'" == "all" | "`stat'" == ""{	
		matrix obs = hr_add1
		quietly bstat hr, stat(obs) n(`obs')
		estat bootstrap, `ci'
		}
	
	if "`saving'" ~= "" {
		save `saving', replace
		}
	
	restore
	}

end
